"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const axios_1 = (0, tslib_1.__importDefault)(require("axios"));
const lodash_1 = require("lodash");
const utils_1 = require("./utils");
class Storage {
    constructor(config, session) {
        this.currentSession = session;
        this.useCookies = config.useCookies;
        this.httpClient = axios_1.default.create({
            baseURL: config.baseURL,
            timeout: 120 * 1000,
            withCredentials: this.useCookies,
        });
    }
    generateAuthorizationHeader() {
        var _a;
        if (this.useCookies)
            return null;
        const JWTToken = (_a = this.currentSession.getSession()) === null || _a === void 0 ? void 0 : _a.jwt_token;
        if (JWTToken) {
            return {
                Authorization: `Bearer ${JWTToken}`,
            };
        }
        else {
            return null;
        }
    }
    put(path, file, metadata = null, onUploadProgress = undefined) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!path.startsWith('/')) {
                throw new Error('`path` must start with `/`');
            }
            const formData = new FormData();
            formData.append('file', file);
            // todo: handle metadata
            if (metadata !== null) {
                console.warn('Metadata is not yet handled in this version.');
            }
            const upload_res = yield this.httpClient.post(`/storage/o${path}`, formData, {
                headers: Object.assign({ 'Content-Type': 'multipart/form-data' }, this.generateAuthorizationHeader()),
                onUploadProgress,
            });
            return upload_res.data;
        });
    }
    putString(path, data, type = 'raw', metadata = null, onUploadProgress = undefined) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!path.startsWith('/')) {
                throw new Error('`path` must start with `/`');
            }
            let fileData;
            let contentType;
            if (type === 'raw') {
                fileData = (0, utils_1.utf8Bytes)(data);
                contentType =
                    metadata && (0, lodash_1.has)(metadata, 'content-type')
                        ? metadata['content-type']
                        : undefined;
            }
            else if (type === 'data_url') {
                let isBase64 = false;
                const matches = data.match(/^data:([^,]+)?,/);
                if (matches === null) {
                    throw "Data must be formatted 'data:[<mediatype>][;base64],<data>";
                }
                const middle = matches[1] || null;
                if (middle != null) {
                    isBase64 = middle.endsWith(';base64');
                    contentType = isBase64
                        ? middle.substring(0, middle.length - ';base64'.length)
                        : middle;
                }
                const restData = data.substring(data.indexOf(',') + 1);
                fileData = isBase64
                    ? (0, utils_1.base64Bytes)(utils_1.StringFormat.BASE64, restData)
                    : (0, utils_1.percentEncodedBytes)(restData);
            }
            if (!fileData) {
                throw new Error('Unbale to generate file data');
            }
            const file = new File([fileData], 'untitled', { type: contentType });
            // create form data
            const form_data = new FormData();
            form_data.append('file', file);
            const uploadRes = yield this.httpClient.post(`/storage/o${path}`, form_data, {
                headers: Object.assign({ 'Content-Type': 'multipart/form-data' }, this.generateAuthorizationHeader()),
                onUploadProgress,
            });
            return uploadRes.data;
        });
    }
    delete(path) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!path.startsWith('/')) {
                throw new Error('`path` must start with `/`');
            }
            const requestRes = yield this.httpClient.delete(`storage/o${path}`, {
                headers: Object.assign({}, this.generateAuthorizationHeader()),
            });
            return requestRes.data;
        });
    }
    getMetadata(path) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!path.startsWith('/')) {
                throw new Error('`path` must start with `/`');
            }
            const res = yield this.httpClient.get(`storage/m${path}`, {
                headers: Object.assign({}, this.generateAuthorizationHeader()),
            });
            return res.data;
        });
    }
}
exports.default = Storage;
//# sourceMappingURL=Storage.js.map